library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity spi_frame is
	Generic (
			frame_width : INTEGER := 7140;
			
			cs_set : INTEGER := 80;
			cs_duration : INTEGER := 7040;
			cs_polarity : STD_LOGIC := '0';
			
			clk_set : INTEGER := 400;
			clk_cycles : INTEGER := 40;
			clk_duty_activ : INTEGER := 80;
			clk_duty_inactiv : INTEGER := 80;
			clk_polarity : STD_LOGIC := '1';
			
			send_set : INTEGER := 320;
			send_duty : INTEGER := 160;
			send_polarity : STD_LOGIC := '1';
			send_number_of_bits : INTEGER := 40;
			
			receive_number_of_bits : INTEGER := 40;
			receive_polarity : STD_LOGIC := '1'
			);
	
	Port (	clk : in  STD_LOGIC;
			reset : in  STD_LOGIC;
			enable : in  STD_LOGIC;
			spi_p_data_in : IN STD_LOGIC_VECTOR(send_number_of_bits-1 downto 0);
			spi_p_data_out : OUT STD_LOGIC_VECTOR(receive_number_of_bits-1 downto 0);
			spi_cs_out : out STD_LOGIC;
			spi_frame_done : out STD_LOGIC;
			spi_clk_out : out STD_LOGIC;
			spi_s_data_in : in STD_LOGIC;
			spi_s_data_out : out STD_LOGIC
		   );
end spi_frame;

architecture Behavioral of spi_frame is

signal cnt_position : INTEGER := 0;
signal sig_cnt_position : INTEGER := 0;

signal spi_clk_out_temp : STD_LOGIC := '0';
signal spi_cs_out_temp : STD_LOGIC := '0';

COMPONENT spi_cs
	GENERIC (
			cs_set : INTEGER;
			cs_duration : INTEGER;
			cs_polarity : STD_LOGIC
			);
	
    PORT (	clk : in  STD_LOGIC;
			reset : in  STD_LOGIC;
			enable : in  STD_LOGIC;
			position : in INTEGER;
			spi_cs_out : out STD_LOGIC
		   );
END COMPONENT;


COMPONENT spi_clk
	GENERIC (
			clk_set : INTEGER;
			clk_cycles : INTEGER;
			clk_duty_activ : INTEGER;
			clk_duty_inactiv : INTEGER;
			clk_polarity : STD_LOGIC
			);
	
    PORT (	clk : in  STD_LOGIC;
			reset : in  STD_LOGIC;
			enable : in  STD_LOGIC;
			position : in  INTEGER;
			spi_clk_out : out  STD_LOGIC
		   );
END COMPONENT;


COMPONENT spi_send
	GENERIC (
			send_set : INTEGER;
			send_duty : INTEGER;
			send_polarity : STD_LOGIC;
			send_number_of_bits : INTEGER
			);
	
    PORT (	clk : in  STD_LOGIC;
			reset : in  STD_LOGIC;
			enable : in  STD_LOGIC;
			position : in  INTEGER;
			spi_p_data_in : in  STD_LOGIC_VECTOR(send_number_of_bits-1 downto 0);
			spi_s_data_out : out  STD_LOGIC
		   );
END COMPONENT;


COMPONENT spi_receive
	GENERIC (
			receive_number_of_bits : integer;
			receive_polarity : STD_LOGIC;
            cs_polarity : STD_LOGIC
			);
	PORT(
		clk : IN std_logic;
		spi_clk : IN std_logic;
		reset : IN std_logic;
		enable : IN std_logic;
        spi_cs : IN  std_logic;
		spi_s_data_in : IN std_logic;          
		spi_p_data_out : OUT std_logic_vector(receive_number_of_bits-1 downto 0)
		);
END COMPONENT;


begin

Inst_SPI_cs : spi_cs
   GENERIC MAP(
      cs_set => cs_set,
      cs_duration => cs_duration,
      cs_polarity => cs_polarity
   )
   PORT MAP(
		clk => clk,
		reset => reset,
		enable => enable,
		position => sig_cnt_position,
		spi_cs_out => spi_cs_out_temp
	);


Inst_SPI_clk : spi_clk
   GENERIC MAP(
      clk_set => clk_set,
      clk_cycles => clk_cycles,
      clk_duty_activ => clk_duty_activ,
	  clk_duty_inactiv => clk_duty_inactiv,
	  clk_polarity => clk_polarity
   )
   PORT MAP(
		clk => clk,
		reset => reset,
		enable => enable,
		position => sig_cnt_position,
		spi_clk_out => spi_clk_out_temp
	);


Inst_SPI_send : spi_send
   GENERIC MAP(
      send_set => send_set,
      send_duty => send_duty,
	  send_polarity => send_polarity,
	  send_number_of_bits => send_number_of_bits
   )
   PORT MAP(
		clk => clk,
		reset => reset,
		enable => enable,
		position => sig_cnt_position,
		spi_p_data_in => spi_p_data_in,
		spi_s_data_out => spi_s_data_out
	);


Inst_spi_receive: spi_receive 
   GENERIC MAP(
		receive_number_of_bits  => receive_number_of_bits,
		receive_polarity  => receive_polarity,
        cs_polarity => cs_polarity
   )
	PORT MAP(
		clk => clk,
		spi_clk => spi_clk_out_temp,
		reset => reset,
		enable => enable,
        spi_cs => spi_cs_out_temp,
		spi_s_data_in => spi_s_data_in,
		spi_p_data_out => spi_p_data_out
	);


	
count_frame : process(clk, reset, enable)
begin
	if clk'event and clk = '1' then
		if reset = '1' OR enable = '0' then -- synchronous reset
		cnt_position <= 0;
        spi_frame_done <= '0';
		else
			if cnt_position = frame_width then -- end of SPI-frame reached
			cnt_position <= 0;
            spi_frame_done <= '1';
			else -- SPI-frame still active
			cnt_position <= cnt_position + 1;
            spi_frame_done <= '0';
			end if;
		end if;
	end if;
end process;

sig_cnt_position <= cnt_position;
spi_clk_out <= spi_clk_out_temp;
spi_cs_out <= spi_cs_out_temp;

end Behavioral;

